var addProps = require("../src/addProps");
var styles = require("../src/defaultStyles");

var padding = {
	top: 0,
	bottom: 4,
	left: 5,
	right: 6
};

var elem = {
	style: {
		paddingBottom: 0,
		paddingTop: 0,
		paddingLeft: 0,
		paddingRight: 0,
		marginBottom: 0,
		marginTop: 0,
		marginLeft: 0,
		marginRight: 0,
		borderTopWidth: 0,
		borderTopStyle: "solid",
		borderTopColor: "#000000",
		borderBottomWidth: 0,
		borderBottomStyle: "solid",
		borderBottomColor: "#000000",
		borderLeftWidth: 0,
		borderLeftStyle: "solid",
		borderLeftColor: "#000000",
		borderRightWidth: 0,
		borderRightStyle: "solid",
		borderRightColor: "#000000",
		borderRadius: 0,
		backgroundColor: "#000000",
		backgroundImage: "",
		backgroundRepeat: "none",
		backgroundPosition: "top left"
	},
	classList: {
		add: function () {}
	}
};

describe("Adding padding", function() {
	addProps.addPadding(elem, padding);
	it("Add", function() {
		expect(elem.style.paddingBottom).toEqual("4px");
	});
	it("And to other", function() {
		expect(elem.style.paddingTop).toEqual(0);
	});
});

describe("Adding margin", function() {
	var margin = {
		top: 13,
		bottom: 14,
		left: 15,
		right: 0
	};
	addProps.addMargin(elem, margin);
	it("Add", function() {
		expect(elem.style.marginLeft).toEqual("15px");
	});
	it("And to other", function() {
		expect(elem.style.marginRight).toEqual(0);
	});
});

describe("Adding border", function() {
	var border = {
		left: {
			width: 0,
			style: "solid",
			color: "#445566"
		},
		top: {
			width: 2,
			style: "solid",
			color: "#AAAAAA"
		},
		right: {
			width: 3,
			style: "solid",
			color: "#445566"
		},
		bottom: {
			width: 4,
			style: "dotted",
			color: "#667788"
		}
	};
	it("Add without radius, with left side of zero", function() {
		addProps.addBorder(elem, border);
		expect(elem.style.borderLeftWidth).toEqual(0);
		expect(elem.style.borderLeftStyle).toEqual("solid");
		expect(elem.style.borderLeftColor).toEqual("#000000");
		expect(elem.style.borderRightStyle).toEqual("solid");
		expect(elem.style.borderTopColor).toEqual("#AAAAAA");
		expect(elem.style.borderTopWidth).toEqual("2px");
		expect(elem.style.borderBottomStyle).toEqual("dotted");
		expect(elem.style.borderRightColor).toEqual("#445566");
		expect(elem.style.borderRightWidth).toEqual("3px");
		expect(elem.style.borderBottomColor).toEqual("#667788");
		expect(elem.style.borderBottomWidth).toEqual("4px");
		expect(elem.style.borderTopStyle).toEqual("solid");
		expect(elem.style.borderRadius).toEqual(0);
	});
	it("Add without radius", function() {
		border.left.width = 1;
		addProps.addBorder(elem, border);
		expect(elem.style.borderLeftWidth).toEqual("1px");
		expect(elem.style.borderRightStyle).toEqual("solid");
		expect(elem.style.borderTopColor).toEqual("#AAAAAA");
		expect(elem.style.borderTopWidth).toEqual("2px");
		expect(elem.style.borderBottomStyle).toEqual("dotted");
		expect(elem.style.borderRightColor).toEqual("#445566");
		expect(elem.style.borderRightWidth).toEqual("3px");
		expect(elem.style.borderLeftStyle).toEqual("solid");
		expect(elem.style.borderBottomColor).toEqual("#667788");
		expect(elem.style.borderBottomWidth).toEqual("4px");
		expect(elem.style.borderTopStyle).toEqual("solid");
		expect(elem.style.borderLeftColor).toEqual("#445566");
		expect(elem.style.borderRadius).toEqual(0);
	});
	it("Add with radius", function() {
		border.radius = 5;
		addProps.addBorder(elem, border);
		expect(elem.style.borderLeftWidth).toEqual("1px");
		expect(elem.style.borderRightStyle).toEqual("solid");
		expect(elem.style.borderTopColor).toEqual("#AAAAAA");
		expect(elem.style.borderTopWidth).toEqual("2px");
		expect(elem.style.borderBottomStyle).toEqual("dotted");
		expect(elem.style.borderRightColor).toEqual("#445566");
		expect(elem.style.borderRightWidth).toEqual("3px");
		expect(elem.style.borderLeftStyle).toEqual("solid");
		expect(elem.style.borderBottomColor).toEqual("#667788");
		expect(elem.style.borderBottomWidth).toEqual("4px");
		expect(elem.style.borderTopStyle).toEqual("solid");
		expect(elem.style.borderLeftColor).toEqual("#445566");
		expect(elem.style.borderRadius).toEqual("5px");
	});
});

describe("Adding background", function() {
	var complexBackground = {
		color: "#ABABAB",
		image: {
			src: "http://foo.com/foo",
			repeat: "repeat-x",
			position: "top center"
		}
	};
	var backgroundColor = "#CCAACC";
	var complexBackgroundBadColor = {
		color: "#ABABABO",
		image: {
			src: "http://foo.com/foo",
			repeat: "repeat-x",
			position: "top left"
		}
	};
	var complexBackgroundWithMissingParams = {
		color: "#BABABA",
		image: {
			src: "",
			repeat: "",
			position: ""
		}
	};
	var backgroundColorBadColor = "#CCAAKK";
	it("Add complex background with no image", function() {
		addProps.addBackground(elem, complexBackgroundWithMissingParams);
		expect(elem.style.backgroundColor).toEqual("#BABABA");
		expect(elem.style.backgroundImage).toEqual("");
		expect(elem.style.backgroundRepeat).toEqual("none");
		expect(elem.style.backgroundPosition).toEqual("top left");
	});
	it("Add complex background with image but with no repeat", function() {
		complexBackgroundWithMissingParams.image.src = "http://foo.com/foo2";
		addProps.addBackground(elem, complexBackgroundWithMissingParams);
		expect(elem.style.backgroundColor).toEqual("#BABABA");
		expect(elem.style.backgroundImage).toEqual("url(http://foo.com/foo2)");
		expect(elem.style.backgroundRepeat).toEqual("none");
		expect(elem.style.backgroundPosition).toEqual("top left");
	});
	it("Add complex background", function() {
		addProps.addBackground(elem, complexBackground);
		expect(elem.style.backgroundColor).toEqual("#ABABAB");
		expect(elem.style.backgroundImage).toEqual("url(http://foo.com/foo)");
		expect(elem.style.backgroundRepeat).toEqual("repeat-x");
		expect(elem.style.backgroundPosition).toEqual("top center");
	});
	it("Add just color", function() {
		addProps.addBackgroundColor(elem, backgroundColor);
		expect(elem.style.backgroundColor).toEqual("#CCAACC");
	});
	it("Do not change complext BG to illegal color", function() {
		addProps.addBackground(elem, complexBackgroundBadColor);
		expect(elem.style.backgroundColor).toEqual("#CCAACC");
	});
	it("Do not change BG to illegal color", function() {
		addProps.addBackgroundColor(elem, backgroundColorBadColor);
		expect(elem.style.backgroundColor).toEqual("#CCAACC");
	});
});

describe("Add divider line", function() {
	var dividerLine = {
		style: "dotted",
		width: 0,
		color: "#B0B0B0"
	};
	it("Do not add if the width is zero", function() {
		var originalValues = {
			borderBottomWidth: elem.style.borderBottomWidth,
			borderBottomStyle: elem.style.borderBottomStyle,
			borderBottomColor: elem.style.borderBottomColor
		};
		addProps.addDividerLine(elem, dividerLine);
		expect(elem.style.borderBottomWidth).toEqual(originalValues.borderBottomWidth);
		expect(elem.style.borderBottomStyle).toEqual(originalValues.borderBottomStyle);
		expect(elem.style.borderBottomColor).toEqual(originalValues.borderBottomColor);
	});
	it("Add if its correct", function() {
		dividerLine.width = 10;
		addProps.addDividerLine(elem, dividerLine);
		expect(elem.style.borderBottomWidth).toEqual("10px");
		expect(elem.style.borderBottomStyle).toEqual("dotted");
		expect(elem.style.borderBottomColor).toEqual("#B0B0B0");
	});
});

describe("Adding typography", function() {
	var typography = {
		text: {
			lineHeight: 14,
			color: "#111111",
			size: 20,
			family: "Arial, sans-serif"
		},
		h1: {
			lineHeight: 20,
			color: "#AAA001",
			size: 30,
			family: "Font h1, sans-serif"
		},
		h2: {
			lineHeight: 18,
			color: "#BBB002",
			size: 28,
			family: "Font h2, sans-serif"
		},
		h3: {
			lineHeight: 16,
			color: "#BBB003",
			size: 26,
			family: "Font h3, sans-serif"
		},
		h4: {
			lineHeight: 14,
			color: "#BBB004",
			size: 24,
			family: "Font h4, sans-serif"
		},
		h5: {
			lineHeight: 12,
			color: "#BBB005",
			size: 22,
			family: "Font h5, sans-serif"
		},
		h6: {
			lineHeight: 10,
			color: "#BBB006",
			size: 20,
			family: "Font h6, sans-serif"
		},
		link: {
			color: "#BBB000",
			underline: false
		}
	};
	var prevClassName;
	it("Add the classname", function() {
		spyOn(elem.classList, "add");
		var className = addProps.addTypography(elem, "Simple text", typography);
		expect(prevClassName).not.toEqual(className);
		prevClassName = className;
		expect(elem.classList.add).toHaveBeenCalledWith(className);
	});
	it("Add typography to unformatted text", function() {
		var className = addProps.addTypography(elem, "Simple text", typography);
		prevClassName = className;
		expect(styles.global["." + className].color).toEqual("#111111");
		expect(styles.global["." + className]["line-height"]).toEqual("14px");
		expect(styles.global["." + className]["font-size"]).toEqual("20px");
		expect(styles.global["." + className]["font-family"]).toEqual("Arial, sans-serif");
		expect(styles.global["." + className + " h1"]).toEqual(undefined);
		expect(styles.global["." + className + " h2"]).toEqual(undefined);
		expect(styles.global["." + className + " h3"]).toEqual(undefined);
		expect(styles.global["." + className + " h4"]).toEqual(undefined);
		expect(styles.global["." + className + " h5"]).toEqual(undefined);
		expect(styles.global["." + className + " h6"]).toEqual(undefined);
		expect(styles.global["." + className + " a"]).toEqual(undefined);
	});
	it("Add typography to text with h1", function() {
		var className = addProps.addTypography(elem, "Simple text with a <h1>title</h1>", typography);
		expect(prevClassName).not.toEqual(className);
		prevClassName = className;
		expect(styles.global["." + className].color).toEqual("#111111");
		expect(styles.global["." + className]["line-height"]).toEqual("14px");
		expect(styles.global["." + className]["font-size"]).toEqual("20px");
		expect(styles.global["." + className]["font-family"]).toEqual("Arial, sans-serif");
		expect(styles.global["." + className + " h1"].color).toEqual("#AAA001");
		expect(styles.global["." + className + " h1"]["line-height"]).toEqual("20px");
		expect(styles.global["." + className + " h1"]["font-size"]).toEqual("30px");
		expect(styles.global["." + className + " h1"]["font-family"]).toEqual("Font h1, sans-serif");
		expect(styles.global["." + className + " h2"]).toEqual(undefined);
		expect(styles.global["." + className + " h3"]).toEqual(undefined);
		expect(styles.global["." + className + " h4"]).toEqual(undefined);
		expect(styles.global["." + className + " h5"]).toEqual(undefined);
		expect(styles.global["." + className + " h6"]).toEqual(undefined);
		expect(styles.global["." + className + " a"]).toEqual(undefined);
	});
	it("Add typography to text with h3 and h6", function() {
		var className = addProps.addTypography(elem, "Simple text with a <h6>h6 title</h6> and a <h3>h3 title</h3>", typography);
		expect(prevClassName).not.toEqual(className);
		prevClassName = className;
		expect(styles.global["." + className].color).toEqual("#111111");
		expect(styles.global["." + className]["line-height"]).toEqual("14px");
		expect(styles.global["." + className]["font-size"]).toEqual("20px");
		expect(styles.global["." + className]["font-family"]).toEqual("Arial, sans-serif");
		expect(styles.global["." + className + " h3"].color).toEqual("#BBB003");
		expect(styles.global["." + className + " h3"]["line-height"]).toEqual("16px");
		expect(styles.global["." + className + " h3"]["font-size"]).toEqual("26px");
		expect(styles.global["." + className + " h3"]["font-family"]).toEqual("Font h3, sans-serif");
		expect(styles.global["." + className + " h6"].color).toEqual("#BBB006");
		expect(styles.global["." + className + " h6"]["line-height"]).toEqual("10px");
		expect(styles.global["." + className + " h6"]["font-size"]).toEqual("20px");
		expect(styles.global["." + className + " h6"]["font-family"]).toEqual("Font h6, sans-serif");
		expect(styles.global["." + className + " h1"]).toEqual(undefined);
		expect(styles.global["." + className + " h2"]).toEqual(undefined);
		expect(styles.global["." + className + " h4"]).toEqual(undefined);
		expect(styles.global["." + className + " h5"]).toEqual(undefined);
		expect(styles.global["." + className + " a"]).toEqual(undefined);
	});
	it("Add typography to text with h6 and an non-underlined link", function() {
		var className = addProps.addTypography(elem, "Text with a <h6>h6 title</h6> and a <a>link</a>", typography);
		expect(prevClassName).not.toEqual(className);
		prevClassName = className;
		expect(styles.global["." + className].color).toEqual("#111111");
		expect(styles.global["." + className]["line-height"]).toEqual("14px");
		expect(styles.global["." + className]["font-size"]).toEqual("20px");
		expect(styles.global["." + className]["font-family"]).toEqual("Arial, sans-serif");
		expect(styles.global["." + className + " h6"].color).toEqual("#BBB006");
		expect(styles.global["." + className + " h6"]["line-height"]).toEqual("10px");
		expect(styles.global["." + className + " h6"]["font-size"]).toEqual("20px");
		expect(styles.global["." + className + " h6"]["font-family"]).toEqual("Font h6, sans-serif");
		expect(styles.global["." + className + " a"].color).toEqual("#BBB000");
		expect(styles.global["." + className + " a"]["text-decoration"]).toEqual("none");
		expect(styles.global["." + className + " h1"]).toEqual(undefined);
		expect(styles.global["." + className + " h2"]).toEqual(undefined);
		expect(styles.global["." + className + " h3"]).toEqual(undefined);
		expect(styles.global["." + className + " h4"]).toEqual(undefined);
		expect(styles.global["." + className + " h5"]).toEqual(undefined);
	});
	it("Add typography to text with an underlined link that encapsulates a h2", function() {
		typography.link.underline = true;
		var className = addProps.addTypography(elem, "Text with a <a><h2>h6 title linked</h2></a>", typography);
		expect(prevClassName).not.toEqual(className);
		prevClassName = className;
		expect(styles.global["." + className].color).toEqual("#111111");
		expect(styles.global["." + className]["line-height"]).toEqual("14px");
		expect(styles.global["." + className]["font-size"]).toEqual("20px");
		expect(styles.global["." + className]["font-family"]).toEqual("Arial, sans-serif");
		expect(styles.global["." + className + " h2"].color).toEqual("#BBB002");
		expect(styles.global["." + className + " h2"]["line-height"]).toEqual("18px");
		expect(styles.global["." + className + " h2"]["font-size"]).toEqual("28px");
		expect(styles.global["." + className + " h2"]["font-family"]).toEqual("Font h2, sans-serif");
		expect(styles.global["." + className + " a"].color).toEqual("#BBB000");
		expect(styles.global["." + className + " a"]["text-decoration"]).toEqual("underline");
		expect(styles.global["." + className + " h1"]).toEqual(undefined);
		expect(styles.global["." + className + " h3"]).toEqual(undefined);
		expect(styles.global["." + className + " h4"]).toEqual(undefined);
		expect(styles.global["." + className + " h5"]).toEqual(undefined);
		expect(styles.global["." + className + " h6"]).toEqual(undefined);
	});
});
