var createElementCore = require("../src/createElementCore");

describe("createElement", function() {
	describe("with invalid params", function () {
		var createElement = createElementCore({
			unitFactory: jasmine.createSpy("unitFactory"),
		});
		it("missing parameter", function() {
			expect(function() {
				createElement();
			}).toThrow();
		});
		it("missing JSONnode config parameter", function() {
			expect(function() {
				createElement({
					foo: {}
				});
			}).toThrow();
		});
		it("missing parent config parameter", function() {
			expect(function() {
				createElement({
					JSONnode: {}
				});
			}).toThrow();
		});
		it("missing DOMDocument config parameter", function() {
			expect(function() {
				createElement({
					JSONnode: {},
					parent: {}
				});
			}).toThrow();
		});
		it("missing manageBuild config parameter", function() {
			expect(function() {
				createElement({
					JSONnode: {},
					parent: {},
					DOMDocument: {}
				});
			}).toThrow();
		});
		it("wrong typeof explicitType parameter", function() {
			expect(function() {
				createElement({
					JSONnode: {},
					parent: {},
					DOMDocument: {},
					manageBuild: {},
					explicitType: 3
				});
			}).toThrow();
		});
	});

	describe("with valid params", function() {
		var createElement;
		var fakeUnitFactory = {
			createRoot: jasmine.createSpy("createRoot"),
			createFullWidthContainer: jasmine.createSpy("createFullWidthContainer").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createLeftFullWidthContainer: jasmine.createSpy("createLeftFullWidthContainer").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createRightFullWidthContainer: jasmine.createSpy("createRightFullWidthContainer").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createTwoCellFullWidthContainer: jasmine.createSpy("createTwoCellFullWidthContainer").and.returnValue(mockHtmlElement()),
			createBox: jasmine.createSpy("createBox").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createText: jasmine.createSpy("createText"),
			createImage: jasmine.createSpy("createImage"),
			createButton: jasmine.createSpy("createButton"),
			createMulticolumn: jasmine.createSpy("createMulticolumn").and.returnValue(mockHtmlElement()),
			createColumn: jasmine.createSpy("createColumn"),
			createDivider: jasmine.createSpy("createDivider"),
			createGroup: jasmine.createSpy("createGroup")
		};

		var fakeHelpers = {
			createClassName: jasmine.createSpy(),
			isColor: jasmine.createSpy(),
			manageMulticolProps: jasmine.createSpy(),
			passWidth: jasmine.createSpy(),
			insertFullWidths: jasmine.createSpy()
		};

		beforeEach(function() {
			createElement = createElementCore({
				unitFactory: fakeUnitFactory,
				helpers: fakeHelpers
			});
		});

		var fakeManageBuild = jasmine.createSpy("manageBuild");

		it("create rootHeader with explicitType", function() {
			var mockDoc = newFakeDocument();
			var mJRootHeader = {
				"document": {
					"root": {
						"type": "dummyRoot"
					},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent":null,
						"background":{
							"color":"",
							"image":{
								"src":"https://images-app.edmdesigner.com/5365cd5315069b891089418a/boiwcz3suevdczmo1vpn.png",
								"position":"center",
								"repeat":"repeat",
								"id":"53997229d3f396000094caab"
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				title: "foo",
				description: "foofoo"
			};
			createElement({
				JSONnode: mJRootHeader,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				explicitType: "rootHeader",
				manageBuild: fakeManageBuild
			});
			expect(fakeHelpers.passWidth).toHaveBeenCalledWith(mJRootHeader);
		});

		it("create rootHeader with explicitType", function() {
			var mockDoc = newFakeDocument();
			var mJRootHeader = {
				"document": {
					"root": {
						"type": "dummyRoot"
					},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent":null,
						"background":{
							"color":"",
							"image":{
								"src":"https://images-app.edmdesigner.com/5365cd5315069b891089418a/boiwcz3suevdczmo1vpn.png",
								"position":"center",
								"repeat":"repeat",
								"id":"53997229d3f396000094caab"
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				title: "foo",
				description: "foofoo"
			};
			createElement({
				JSONnode: mJRootHeader,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				explicitType: "rootHeader",
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createRoot).toHaveBeenCalled();
		});

		it("create rootHeader with type", function() {
			var mockDoc = newFakeDocument();
			var mJRootHeader = {
				"type": "rootHeader",
				"document": {
					"root": {
						"type": "dummyRoot"
					},
					"generalSettings":{
						"type":"GENERAL",
						"headerContent":null,
						"background":{
							"color":"",
							"image":{
								"src":"https://images-app.edmdesigner.com/5365cd5315069b891089418a/boiwcz3suevdczmo1vpn.png",
								"position":"center",
								"repeat":"repeat",
								"id":"53997229d3f396000094caab"
							}
						}
					},
					"usedColors": [],
					"header": null,
					"footer": null
				},
				title: "foo",
				description: "foofoo"
			};
			createElement({
				JSONnode: mJRootHeader,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createRoot).toHaveBeenCalled();
		});

		it("create full FullWidth", function() {
			var mockDoc = newFakeDocument();
			var mJFullWidth = {
				type: "FULLWIDTH_CONTAINER",
				order: "LTR",
				twoCell: false,
				leftBackgroundColor: "#AABBAA",
				rightBackgroundColor: "",
				leftChildren: [],
				rightChildren: [],
				isLeftHigher: false,
				hideOnMobile: false
			};
			createElement({
				JSONnode: mJFullWidth,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createFullWidthContainer)
				.toHaveBeenCalledWith(mJFullWidth, mockDoc);
			expect(fakeUnitFactory.createTwoCellFullWidthContainer)
				.not.toHaveBeenCalled();
		});

		it("create twocell FullWidth", function() {
			var mockDoc = newFakeDocument();
			var mJTFullWidth = {
				type: "FULLWIDTH_CONTAINER",
				order: "LTR",
				twoCell: true,
				leftBackgroundColor: "#AABBAA",
				rightBackgroundColor: "#BBAABB",
				leftChildren: [],
				rightChildren: [],
				isLeftHigher: false,
				hideOnMobile: false
			};
			createElement({
				JSONnode: mJTFullWidth,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createTwoCellFullWidthContainer)
				.toHaveBeenCalledWith(mJTFullWidth, mockDoc);
			expect(fakeUnitFactory.createLeftFullWidthContainer)
				.toHaveBeenCalledWith(mJTFullWidth, mockDoc);
			expect(fakeUnitFactory.createRightFullWidthContainer)
				.toHaveBeenCalledWith(mJTFullWidth, mockDoc);
		});

		it("create BOX", function() {
			var mockDoc = newFakeDocument();
			var mJBox = {
				type: "BOX",
				children: []
			};
			createElement({
				JSONnode: mJBox,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createBox)
				.toHaveBeenCalledWith(mJBox, mockDoc);
		});

		it("create GROUP", function() {
			var mockDoc = newFakeDocument();
			var mJGroup = {
				type: "GROUP",
				children: []
			};
			createElement({
				JSONnode: mJGroup,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createGroup)
				.toHaveBeenCalledWith(mJGroup, mockDoc);
		});

		it("create COMPLEX", function() {
			var mockDoc = newFakeDocument();
			var mJComplex = {
				type: "COMPLEX",
				root: {
					type: "BOX",
					children: []
				}
			};
			createElement({
				JSONnode: mJComplex,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createBox)
				.toHaveBeenCalledWith(mJComplex.root, mockDoc);
		});

		it("create DIVIDER", function() {
			var mockDoc = newFakeDocument();
			var mJDivider = {
				type: "DIVIDER"
			};
			createElement({
				JSONnode: mJDivider,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createDivider)
				.toHaveBeenCalledWith(mJDivider, mockDoc);
		});

		it("create MULTICOLUMN", function() {
			var mockDoc = newFakeDocument();
			var mJMulticol = {
				type: "MULTICOLUMN",
				cols: []
			};
			createElement({
				JSONnode: mJMulticol,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createMulticolumn)
				.toHaveBeenCalledWith(mJMulticol, mockDoc);
			expect(fakeHelpers.manageMulticolProps)
				.toHaveBeenCalledWith(mJMulticol);
		});

		it("create MULTICOLUMN and call COLUMN after", function() {
			var mockDoc = newFakeDocument();
			var mJMulticol = {
				type: "MULTICOLUMN",
				cols: [
					{
						width: "50%",
						children: []
					},
					{
						width: "50%",
						children: []
					}
				]
			};
			createElement({
				JSONnode: mJMulticol,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createMulticolumn)
				.toHaveBeenCalledWith(mJMulticol, mockDoc);
			expect(fakeUnitFactory.createColumn)
				.toHaveBeenCalled();
			expect(fakeUnitFactory.createColumn.calls.count()).toEqual(2);
		});

		it("create COLUMN with explicitType", function() {
			var mockDoc = newFakeDocument();
			var mJCol = {
				width: "50%",
				children: []
			};
			createElement({
				JSONnode: mJCol,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				explicitType: "COLUMN",
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createColumn)
				.toHaveBeenCalledWith(mJCol, mockDoc);
		});

		it("create COLUMN with type", function() {
			var mockDoc = newFakeDocument();
			var mJCol = {
				type: "COLUMN",
				width: "50%",
				children: []
			};
			createElement({
				JSONnode: mJCol,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createColumn)
				.toHaveBeenCalledWith(mJCol, mockDoc);
		});

		it("create TEXT", function() {
			var mockDoc = newFakeDocument();
			var mJText = {
				type: "TEXT"
			};
			createElement({
				JSONnode: mJText,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createText)
				.toHaveBeenCalledWith(mJText, mockDoc);
		});

		it("create IMAGE", function() {
			var mockDoc = newFakeDocument();
			var mJImage = {
				type: "IMAGE"
			};
			createElement({
				JSONnode: mJImage,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createImage)
				.toHaveBeenCalledWith(mJImage, mockDoc, fakeManageBuild);
		});

		it("create BUTTON", function() {
			var mockDoc = newFakeDocument();
			var mJButton = {
				type: "BUTTON"
			};
			createElement({
				JSONnode: mJButton,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createButton)
				.toHaveBeenCalledWith(mJButton, mockDoc);
		});

	});

	describe("with valid params, but without type", function() {
		var createElement;
		var fakeUnitFactory = {
			createRoot: jasmine.createSpy("createRoot"),
			createFullWidthContainer: jasmine.createSpy("createFullWidthContainer").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createLeftFullWidthContainer: jasmine.createSpy("createLeftFullWidthContainer").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createRightFullWidthContainer: jasmine.createSpy("createRightFullWidthContainer").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createTwoCellFullWidthContainer: jasmine.createSpy("createTwoCellFullWidthContainer").and.returnValue(mockHtmlElement()),
			createBox: jasmine.createSpy("createBox").and.returnValue({
				outer: mockHtmlElement(),
				inner: mockHtmlElement()
			}),
			createText: jasmine.createSpy("createText"),
			createImage: jasmine.createSpy("createImage"),
			createButton: jasmine.createSpy("createButton"),
			createMulticolumn: jasmine.createSpy("createMulticolumn").and.returnValue(mockHtmlElement()),
			createColumn: jasmine.createSpy("createColumn"),
			createDivider: jasmine.createSpy("createDivider")
		};

		var fakeHelpers = {
			createClassName: jasmine.createSpy(),
			isColor: jasmine.createSpy(),
			manageMulticolProps: jasmine.createSpy(),
			passWidth: jasmine.createSpy()
		};

		var fakeManageBuild = jasmine.createSpy("manageBuild");

		beforeEach(function() {
			createElement = createElementCore({
				unitFactory: fakeUnitFactory,
				helpers: fakeHelpers
			});
		});

		it("do not create anything if no type or explicittype is provided", function() {
			var mockDoc = newFakeDocument();
			var mJNothing = {
				foo: "foo"
			};
			createElement({
				JSONnode: mJNothing,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createRoot)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createLeftFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createRightFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createTwoCellFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createBox)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createText)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createImage)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createButton)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createMulticolumn)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createColumn)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createDivider)
				.not.toHaveBeenCalled();
		});

		it("do not create anything if illegal type is provided", function() {
			var mockDoc = newFakeDocument();
			var mJNothing = {
				type: "RUMKOKOS"
			};
			createElement({
				JSONnode: mJNothing,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createRoot)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createLeftFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createRightFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createTwoCellFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createBox)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createText)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createImage)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createButton)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createMulticolumn)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createColumn)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createDivider)
				.not.toHaveBeenCalled();
		});

		it("do not create anything if illegal explicitType is provided", function() {
			var mockDoc = newFakeDocument();
			var mJNothing = {};
			createElement({
				JSONnode: mJNothing,
				parent: mockDoc.body,
				DOMDocument: mockDoc,
				explicitType: "RUMKOKOS",
				manageBuild: fakeManageBuild
			});
			expect(fakeUnitFactory.createRoot)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createLeftFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createRightFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createTwoCellFullWidthContainer)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createBox)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createText)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createImage)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createButton)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createMulticolumn)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createColumn)
				.not.toHaveBeenCalled();
			expect(fakeUnitFactory.createDivider)
				.not.toHaveBeenCalled();
		});

	});

});

function mockHtmlElement(tag) {
	return {
		style: {
			display: "default"
		},
		id: null,
		tag: tag,
		innerHTML: "",
		appendChild: function() {},
		removeChild: function() {},
		classList: {
			add: function() {},
			remove: function() {}
		}
	};
}

function newFakeDocument(){
	return {
		childNodes: [],
		createElement: function(tag) {
			var newElement = mockHtmlElement(tag);
			this.childNodes.push(newElement);
			return newElement;
		},
		getElementById: function(id) {
			for (var i=0; i<this.childNodes.length; i += 1) {
				if (this.childNodes[i].id === id) {
					return this.childNodes[i];
				}
			}
			return null;
		},
		body: mockHtmlElement("body"),
		head: mockHtmlElement("head"),
		title: ""
	};
}