var subUnitFactory = require("../src/subUnitFactory");

describe("subUnitFactory", function() {

	var mockDoc;
	var fakeManageBuild = jasmine.createSpy("manageBuild");

	beforeEach(function(){
		mockDoc = newFakeDocument();
		spyOn(mockDoc, "createElement").and.callThrough();
	});

	describe("create link element", function() {

		it("create the proper elements", function(){
			var linkElem = subUnitFactory.createLinkElem("http://src", mockDoc);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("a");
			expect(linkElem.href).toEqual("http://src");
		});

	});

	describe("create image element", function() {

		it("create the proper elements", function(){
			var imageElem = subUnitFactory.createImageElem("http://src.jpg", undefined, "left", mockDoc, fakeManageBuild);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("img");
			expect(imageElem.src).toEqual("http://src.jpg");
			expect(imageElem.classList.add)
			.toHaveBeenCalledWith("left-block");
		});

		it("add alt text if present", function(){
			var imageElem = subUnitFactory.createImageElem("http://src.jpg", "altttext", "left", mockDoc, fakeManageBuild);
			expect(imageElem.alt).toEqual("altttext");
		});

		it("manage builds - increase counter", function(){
			subUnitFactory.createImageElem("http://src.jpg", "altttext", "left", mockDoc, fakeManageBuild);
			expect(fakeManageBuild)
			.toHaveBeenCalledWith(1);
		});

		it("manage builds - set decreasement in onload", function(){
			var imageElem = subUnitFactory.createImageElem("http://src.jpg", "altttext", "left", mockDoc, fakeManageBuild);
			expect(imageElem.onload)
			.toEqual(jasmine.any(Function));
		});

		it("manage builds - onload should fire the manageBuild", function(){
			var imageElem = subUnitFactory.createImageElem("http://src.jpg", "altttext", "left", mockDoc, fakeManageBuild);
			imageElem.onload();
			expect(fakeManageBuild)
			.toHaveBeenCalledWith(-1);
		});

	});

	describe("create text element", function() {

		it("create the proper elements", function(){
			var textElem = subUnitFactory.createTextElem("FOO", mockDoc);
			expect(mockDoc.createElement)
			.toHaveBeenCalledWith("div");
			expect(textElem.innerHTML).toEqual("FOO");
		});

	});

});

function mockHtmlElement(tag) {
	return {
		style: {
			display: "default"
		},
		id: null,
		tag: tag,
		innerHTML: "",
		appendChild: jasmine.createSpy(),
		removeChild: jasmine.createSpy(),
		classList: {
			add: jasmine.createSpy(),
			remove: jasmine.createSpy()
		}
	};
}

function newFakeDocument(){
	return {
		childNodes: [],
		createElement: function(tag) {
			var newElement = mockHtmlElement(tag);
			this.childNodes.push(newElement);
			return newElement;
		},
		getElementById: function(id) {
			for (var i=0; i<this.childNodes.length; i += 1) {
				if (this.childNodes[i].id === id) {
					return this.childNodes[i];
				}
			}
			return null;
		},
		body: mockHtmlElement("body"),
		head: mockHtmlElement("head"),
		title: ""
	};
}
