module.exports = {
	addPadding: addPadding,
	addMargin: addMargin,
	addDividerLine: addDividerLine,
	addBorderItem: addBorderItem,
	addBorder: addBorder,
	addTypography: addTypography,
	addBackground: addBackground,
	addBackgroundColor: addBackgroundColor
};

var styles = require("./defaultStyles");
var helpers = require("./helpers");

var sides = {
	"left": "Left",
	"right": "Right",
	"top": "Top",
	"bottom": "Bottom"
};

function camelize(originalString) {
	return sides[originalString];
}

function addPadding(elem, padding) {
	for (var side in sides) {
		if (padding[side] > 0) {
			elem.style["padding" + sides[side]] = padding[side] + "px";
		}
	}
}

function addMargin(elem, margin) {
	for (var side in sides) {
		if (margin[side] > 0) {
			elem.style["margin" + sides[side]] = margin[side] + "px";
		}
	}
}

function addDividerLine(elem, lineStyle) {
	if (lineStyle.width > 0) {
		elem.style.borderBottomColor = lineStyle.color;
		elem.style.borderBottomWidth = lineStyle.width + "px";
		elem.style.borderBottomStyle = lineStyle.style;
	}
}

function addBorderItem(elem, border, side) {
	var sSide = camelize(side);
	if (border[side].width > 0) {
		elem.style["border" + sSide + "Color"] = border[side].color;
		elem.style["border" + sSide + "Width"] = border[side].width + "px";
		elem.style["border" + sSide + "Style"] = border[side].style;
	}
}

function addBorder(elem, border) {
	for (var side in sides) {
		addBorderItem(elem, border, side);
	}
	if (border.radius) {
		elem.style.borderRadius = border.radius + "px";
	}
}

function addTypography(elem, content, typography) {
	var tags = ["h1", "h2", "h3", "h4", "h5", "h6"];
	var className = helpers.createClassName("text-box-");
	styles.global["." + className] = {
		"line-height": typography.text.lineHeight + "px",
		"color": typography.text.color,
		"font-size" : typography.text.size + "px",
		"font-family": typography.text.family
	};
	for (var i = 0; i < tags.length; i += 1) {
		if (typography[tags[i]] && (new RegExp("<" + tags[i])).test(content)) {
			styles.global["." + className + " " + tags[i]] = {
				"line-height": typography[tags[i]].lineHeight + "px",
				"color": typography[tags[i]].color,
				"font-size" : typography[tags[i]].size + "px",
				"font-family": typography[tags[i]].family
			};
		}
		if(typography.link && /<a/i.test(content)) {
			var underline = "none";
			if (typography.link.underline && typography.link.underline !== "false") {
				underline = "underline";
			}
			styles.global["." + className + " a"] = {
				"color": typography.link.color,
				"text-decoration": underline
			};
		}
	}
	elem.classList.add(className);
	return className;
}

function addBackground(elem, background) {
	if (helpers.isColor(background.color)) {
		elem.style.backgroundColor = background.color;
	}
	if (background.image && background.image.src && background.image.src !== "") {
		elem.style.backgroundImage = "url(" + background.image.src + ")";
		if (background.image.repeat && background.image.repeat !== "") {
			elem.style.backgroundRepeat = background.image.repeat;
		}
		if (background.image.position && background.image.position !== "") {
			elem.style.backgroundPosition = background.image.position;
		}
	}
}

function addBackgroundColor(elem, colorCandidate) {
	if (helpers.isColor(colorCandidate)) {
		elem.style.backgroundColor = colorCandidate;
	}
}
