module.exports = function(dependencies) {
	var unitFactory = dependencies.unitFactory;
	var helpers = dependencies.helpers;

	return function createElement(params) {
		if (!params) {
			params = {};
		}
		if (!params.JSONnode) {
			throw new Error("JSONnode param is mandarory!");
		}
		if (!params.parent) {
			throw new Error("parent param is mandarory!");
		}
		if (!params.DOMDocument) {
			throw new Error("DOMDocument param is mandarory!");
		}
		if (params.explicitType && typeof params.explicitType !== "string") {
			throw new Error("explicitType param must be String, the given parameter is " + typeof params.explicitType + "!");
		}
		if (!params.manageBuild) {
			throw new Error("manageBuild param is mandarory!");
		}
		var JSONnode = params.JSONnode;
		var parent = params.parent;
		var DOMDocument = params.DOMDocument;
		var explicitType = params.explicitType;
		var manageBuild = params.manageBuild;

		var defaultBodyBackground = params.defaultBodyBackground;

		var elementToAddToParent;
		var nextExplicitType;
		var inheritence = [];

		helpers.passWidth(JSONnode);

		manageBuild(1);

		if (explicitType) {
			if (explicitType === "COLUMN") {
				// console.log("creating COLUMN");
				elementToAddToParent = unitFactory.createColumn(JSONnode, DOMDocument);
				inheritence.push({
					nextParent: elementToAddToParent,
					children: JSONnode.children
				});
			} else if (explicitType === "rootHeader" && JSONnode.document.root) {
				// console.log("creating rootHeader");
				helpers.insertFullWidths(JSONnode.document.root);
				elementToAddToParent = unitFactory.createRoot(JSONnode, DOMDocument, defaultBodyBackground);
				inheritence.push({
					nextParent: elementToAddToParent,
					children: JSONnode.document.root.children
				});
			}
		} else {
			switch(JSONnode.type) {
				case "rootHeader": {
					// console.log("creating rootHeader");
					helpers.insertFullWidths(JSONnode.document.root);
					elementToAddToParent = unitFactory.createRoot(JSONnode, DOMDocument, defaultBodyBackground);
					inheritence.push({
						nextParent: elementToAddToParent,
						children: JSONnode.document.root.children
					});
					break;
				}
				case "FULLWIDTH_CONTAINER": {
					if (JSONnode.twoCell && JSONnode.twoCell !== "false") {
						// console.log("creating twoCell FULLWIDTH_CONTAINER");
						var wrapperElem = unitFactory.createTwoCellFullWidthContainer(JSONnode, DOMDocument);
						elementToAddToParent = wrapperElem;

						var leftFw = unitFactory.createLeftFullWidthContainer(JSONnode, DOMDocument);
						wrapperElem.appendChild(leftFw.outer);
						inheritence.push({
							nextParent: leftFw.inner,
							children: JSONnode.leftChildren
						});

						var rightFw = unitFactory.createRightFullWidthContainer(JSONnode, DOMDocument);
						wrapperElem.appendChild(rightFw.outer);
						inheritence.push({
							nextParent: rightFw.inner,
							children: JSONnode.rightChildren
						});
					} else {
						// console.log("creating oneCell FULLWIDTH_CONTAINER");
						var fw = unitFactory.createFullWidthContainer(JSONnode, DOMDocument);
						elementToAddToParent = fw.outer;
						inheritence.push({
							nextParent: fw.inner,
							children: JSONnode.leftChildren
						});
					}
					break;
				}
				case "BOX": {
					// console.log("creating BOX");
					var newBox = unitFactory.createBox(JSONnode, DOMDocument);
					elementToAddToParent = newBox.outer;
					inheritence.push({
						nextParent: newBox.inner,
						children: JSONnode.children
					});
					break;
				}
				case "GROUP": {
					console.log("creating GROUP");
					// add group as a spacingless, backgroundless boxlike wrapper
					var newGroup = unitFactory.createGroup(JSONnode, DOMDocument);
					elementToAddToParent = newGroup;
					inheritence.push({
						nextParent: newGroup,
						children: JSONnode.children
					});
					break;
				}
				case "COMPLEX": {
					// console.log("creating COMPLEX of subType " + JSONnode.subType);
					var newBox = unitFactory.createBox(JSONnode.root, DOMDocument);
					elementToAddToParent = newBox.outer;
					inheritence.push({
						nextParent: newBox.inner,
						children: JSONnode.root.children
					});
					break;
				}
				case "DIVIDER": {
					// console.log("creating DIVIDER");
					elementToAddToParent = unitFactory.createDivider(JSONnode, DOMDocument);
					break;
				}
				case "MULTICOLUMN": {
					// console.log("creating MULTICOLUMN");
					var elementToAddToParent = unitFactory.createMulticolumn(JSONnode, DOMDocument);
					helpers.manageMulticolProps(JSONnode);
					inheritence.push({
						nextParent: elementToAddToParent,
						children: JSONnode.cols
					});
					nextExplicitType = "COLUMN";
					break;
				}
				case "COLUMN": {
					// console.log("creating COLUMN");
					elementToAddToParent = unitFactory.createColumn(JSONnode, DOMDocument);
					inheritence.push({
						nextParent: elementToAddToParent,
						children: JSONnode.children
					});
					break;
				}
				case "TEXT": {
					// console.log("creating TEXT");
					elementToAddToParent = unitFactory.createText(JSONnode, DOMDocument);
					break;
				}
				case "IMAGE": {
					// console.log("creating IMAGE");
					elementToAddToParent = unitFactory.createImage(JSONnode, DOMDocument, manageBuild);
					break;
				}
				case "BUTTON": {
					// console.log("creating BUTTON");
					elementToAddToParent = unitFactory.createButton(JSONnode, DOMDocument);
					break;
				}
				default: {
					// console.log("Could not create element from type " + JSONnode.type);
					break;
				}
			}
		}

		if (elementToAddToParent) {
			parent.appendChild(elementToAddToParent);
			for (var j = 0; j < inheritence.length; j += 1) {
				for (var i = 0; i < inheritence[j].children.length; i += 1) {
					createElement({
						JSONnode: inheritence[j].children[i],
						parent: inheritence[j].nextParent,
						DOMDocument: DOMDocument,
						explicitType: nextExplicitType,
						manageBuild: manageBuild
					});
				}
			}
		}

		manageBuild(-1);

	};
};
