module.exports = function(dependencies) {
	var renderCss = dependencies.renderCss;
	var createElement = dependencies.createElement;

	return function generateHTML(config) {

		var toFinish = 0;

		function manageBuild(offset) {
			toFinish += offset;
			if (toFinish === 0 && config.afterBuild) {
				config.afterBuild();
			}
		}

		if (!config) {
			config = {};
		}

		if (!config.DOMDocument) {
			throw new Error("DOMDocument is mandatory!");
		}

		if (!config.mailJSON) {
			// throw new Error("config.mailJSON is mandatory!");
			// console.log("Clearing");
			var placeholder = config.DOMDocument.getElementById("placeholder");
			if (placeholder){
				placeholder.style.display = "block";
			}
			if (config.DOMDocument.getElementById("rootHeader")){
				config.DOMDocument.body.removeChild(config.DOMDocument.getElementById("rootHeader"));
			}
			if (config.DOMDocument.getElementById("styleTag")){
				config.DOMDocument.head.removeChild(config.DOMDocument.getElementById("styleTag"));
			}
			if (config.afterBuild && typeof config.afterBuild === "function") {
				config.afterBuild();
			}
			return;
		}

		var defaultBodyBackground = false;
		if (config.defaultBodyBackground) {
			defaultBodyBackground = config.defaultBodyBackground;
		}

		var mailJSON = config.mailJSON;
		var DOMDocument = config.DOMDocument;
		var scroll = false;
		if (config.scroll) {
			scroll = true;
		}

		// console.log("Starting generation");
		var placeholder = DOMDocument.getElementById("placeholder");
		if (placeholder){
			placeholder.style.display = "none";
		}
		if (DOMDocument.getElementById("rootHeader")){
			DOMDocument.body.removeChild(DOMDocument.getElementById("rootHeader"));
		}
		if (config.DOMDocument.getElementById("styleTag")){
			config.DOMDocument.head.removeChild(config.DOMDocument.getElementById("styleTag"));
		}
		createElement({
			JSONnode: mailJSON,
			defaultBodyBackground: defaultBodyBackground,
			parent: DOMDocument.body,
			DOMDocument: DOMDocument,
			explicitType: "rootHeader",
			manageBuild: manageBuild
		});
		var styleTag = DOMDocument.createElement("style");
		styleTag.id = "styleTag";
		styleTag.innerHTML = renderCss(scroll);
		DOMDocument.head.appendChild(styleTag);
	};
};
