module.exports = {
	createClassName: createClassName,
	isColor: isColor,
	manageMulticolProps: manageMulticolProps,
	passWidth: passWidth,
	insertFullWidths: insertFullWidths,
	handleOldTypography: handleOldTypography
};

var styleCounter = 0;
var boxSize = 600;

function createClassName(base) {
	styleCounter += 1;
	return base + styleCounter;
}

function isColor(str) {
	if (!str || typeof str !== "string") {
		return false;
	}
	if (/^\s*#[a-f0-9]{6}\s*$/i.test(str)) {
		return true;
	}
	return false;
}

function manageMulticolProps(JSONnode) {
	if (JSONnode.type !== "MULTICOLUMN") {
		return false;
	}
	var hiddenPercentage = 0;
	for (var i = 0; i < JSONnode.cols.length; i += 1) {
		JSONnode.cols[i].reorderOnMobile = JSONnode.reorderOnMobile || false;
		JSONnode.cols[i].diffOnMobile = false;
		if (JSONnode.cols[i].hideOnMobile && JSONnode.cols[i].hideOnMobile !== "false") {
			hiddenPercentage += JSONnode.cols[i].width;
		}
	}
	if (JSONnode.reorderOnMobile === true || JSONnode.reorderOnMobile === "true") {
		return false;
	}
	if (hiddenPercentage > 0) {
		var remainingPercentage = 100 - hiddenPercentage;
		for (var i = 0; i < JSONnode.cols.length; i += 1) {
			JSONnode.cols[i].diffOnMobile = true;
			if (JSONnode.cols[i].hideOnMobile && JSONnode.cols[i].hideOnMobile !== "false") {
				JSONnode.cols[i].mobileWidth = 0;
			} else {
				JSONnode.cols[i].mobileWidth = 100*(JSONnode.cols[i].width / remainingPercentage);
			}
		}
	}
}

function passWidth(JSONnode) {
	if (!JSONnode || !JSONnode.type) {
		return false;
	}
	if (JSONnode.type !== "BOX" &&
		JSONnode.type !== "FULLWIDTH_CONTAINER" &&
		JSONnode.type !== "MULTICOLUMN" &&
		JSONnode.type !== "COMPLEX") {
		return false;
	}
	if (JSONnode.type !== "FULLWIDTH_CONTAINER" && !JSONnode.computedWith) {
		console.log("Element " + JSONnode.type + " did not recieve computedWith");
		return false;
	}
	switch(JSONnode.type) {
		case "FULLWIDTH_CONTAINER": {
			JSONnode.computedWith = boxSize;
			if (JSONnode.twoCell && JSONnode.twoCell !== "false") {
				for (var i = 0; i < JSONnode.leftChildren.length; i +=1) {
					JSONnode.leftChildren[i].computedWith = boxSize/2;
				}
				for (var i = 0; i < JSONnode.rightChildren.length; i +=1) {
					JSONnode.rightChildren[i].computedWith = boxSize/2;
				}
			} else {
				for (var i = 0; i < JSONnode.leftChildren.length; i +=1) {
					JSONnode.leftChildren[i].computedWith = boxSize;
				}
			}
			break;
		}
		case "BOX": {
			var chWidth = JSONnode.computedWith -
				JSONnode.margin.left - JSONnode.margin.right -
				JSONnode.padding.left - JSONnode.padding.right -
				JSONnode.border.left.width - JSONnode.border.right.width;
			for (var i = 0; i < JSONnode.children.length; i +=1) {
				JSONnode.children[i].computedWith = chWidth;
			}
			break;
		}
		case "COMPLEX": {
			var chWidth = JSONnode.computedWith -
				JSONnode.root.margin.left - JSONnode.root.margin.right -
				JSONnode.root.padding.left - JSONnode.root.padding.right -
				JSONnode.root.border.left.width - JSONnode.root.border.right.width;
			for (var i = 0; i < JSONnode.root.children.length; i +=1) {
				JSONnode.root.children[i].computedWith = chWidth;
			}
			break;
		}
		case "MULTICOLUMN": {
			if (JSONnode.computedWith < 240) {
				JSONnode.reorderOnMobile = false;
			}
			for (var i = 0; i < JSONnode.cols.length; i += 1) {
				var chWidth = JSONnode.computedWith * JSONnode.cols[i].width / 100;
				for (var j = 0; j < JSONnode.cols[i].children.length; j += 1) {
					JSONnode.cols[i].children[j].computedWith = chWidth;
				}
			}
			break;
		}
	}
}

function insertFullWidths(node) {

	if (node.type !== "ROOT" && node.explicitType !== "ROOT") {
		return false;
	}

	if (!node.children) {
		return false;
	}

	for (var i = 0; i < node.children.length; i += 1) {
		if (node.children[i].type === "GROUP") {
			for (var j = 0; j < node.children[i].children.length; j += 1) {
				if (node.children[i].children[j].type !== "FULLWIDTH_CONTAINER") {
					var newFullWidth = {
						"type": "FULLWIDTH_CONTAINER",
						"customData": {},
						"order": "LTR",
						"twoCell": false,
						"leftBackgroundColor": "",
						"rightBackgroundColor": "",
						"leftChildren": [node.children[i].children[j]],
						"rightChildren": [],
						"isLeftHigher": true,
						"hideOnMobile": false
					};
					node.children[i].children[j] = newFullWidth;
				}
			}
			continue;
		}
		if (node.children[i].type !== "FULLWIDTH_CONTAINER") {
			var newFullWidth = {
				"type": "FULLWIDTH_CONTAINER",
				"customData": {},
				"order": "LTR",
				"twoCell": false,
				"leftBackgroundColor": "",
				"rightBackgroundColor": "",
				"leftChildren": [node.children[i]],
				"rightChildren": [],
				"isLeftHigher": true,
				"hideOnMobile": false
			};
			node.children[i] = newFullWidth;
		}
	}
}

function handleOldTypography(JSONnode) {
	if (JSONnode.typography) {
		return;
	}

	var textStyles = JSONnode.textStyles || {
		color: "#000001",
		size: 14,
		family: "arial",
		lineHeight: 14
	};

	JSONnode.typography = {
		text: textStyles,
		h1: JSONnode.h1Styles || textStyles,
		h2: JSONnode.h2Styles || textStyles,
		h3: JSONnode.h3Styles || textStyles,
		h4: JSONnode.h4Styles || textStyles,
		h5: JSONnode.h5Styles || textStyles,
		h6: JSONnode.h6Styles || textStyles
	};
}
