

module.exports = function(dependencies) {
	var styles = dependencies.styles;
	var addProps = dependencies.addProps;
	var helpers = dependencies.helpers;
	var subUnitFactory = dependencies.subUnitFactory;
	var extend = dependencies.extend;

	var defaultGeneralSettings = {
		background: {
			color: "#FFFFFF"
		}
	};

	function createRoot(JSONnode, DOMDocument, defaultBodyBackground) {
		DOMDocument.title = JSONnode.title;
		var elem = DOMDocument.createElement("div");
		elem.classList.add("elem-root", "elem-inspector", "selenium-json-preview-root");
		elem.id = "rootHeader";
		JSONnode.document.generalSettings = extend(true, {}, defaultGeneralSettings, JSONnode.document.generalSettings);


		var background = JSONnode.document.generalSettings.background;
		if (helpers.isColor(background.color)) {
			styles.global.body.background = background.color;
		} else {
			styles.global.body.background = defaultBodyBackground || "#FFFFFF";
		}
		if (background.image && background.image.src && background.image.src !== "") {
			styles.global.body["background-image"] = "url(" + background.image.src + ")";
			if (background.image.repeat && background.image.repeat !== "") {
				styles.global.body["background-repeat"] = background.image.repeat;
			}
			if (background.image.position && background.image.position !== "") {
				styles.global.body["background-position"] = background.image.position;
			}
		}
		return elem;
	}

	function createFullWidthContainer(JSONnode, DOMDocument) {
		var innerElem = DOMDocument.createElement("div");
		innerElem.classList.add("full-width-inner");
		var wrapperElem = DOMDocument.createElement("div");
		wrapperElem.classList.add("elem-fullwidth-onecell", "elem-inspector");
		addProps.addBackgroundColor(wrapperElem, JSONnode.leftBackgroundColor);
		wrapperElem.classList.add("full-width-wrapper");
		wrapperElem.appendChild(innerElem);
		if (JSONnode.hideOnMobile && JSONnode.hideOnMobile !== "false") {
			innerElem.classList.add("hide-on-tablet");
		}
		return {
			outer: wrapperElem,
			inner: innerElem
		};
	}

	function createLeftFullWidthContainer(JSONnode, DOMDocument) {
		var innerElem = DOMDocument.createElement("div");
		innerElem.classList.add("left-full-width-inner");
		var wrapperElem = DOMDocument.createElement("div");
		wrapperElem.classList.add("elem-fullwidth-twocell-left", "elem-inspector");
		addProps.addBackgroundColor(wrapperElem, JSONnode.leftBackgroundColor);
		wrapperElem.classList.add("left-full-width-wrapper");
		wrapperElem.appendChild(innerElem);
		return {
			outer: wrapperElem,
			inner: innerElem
		};
	}

	function createRightFullWidthContainer(JSONnode, DOMDocument) {
		var innerElem = DOMDocument.createElement("div");
		innerElem.classList.add("right-full-width-inner");
		var wrapperElem = DOMDocument.createElement("div");
		wrapperElem.classList.add("elem-fullwidth-twocell-right", "elem-inspector");
		addProps.addBackgroundColor(wrapperElem, JSONnode.rightBackgroundColor);
		wrapperElem.classList.add("right-full-width-wrapper");
		wrapperElem.appendChild(innerElem);
		return {
			outer: wrapperElem,
			inner: innerElem
		};
	}

	function createTwoCellFullWidthContainer(JSONnode, DOMDocument) {
		var elem = DOMDocument.createElement("div");
		if (JSONnode.hideOnMobile && JSONnode.hideOnMobile !== "false") {
			elem.classList.add("hide-on-tablet");
		}
		return elem;
	}

	function createGroup(JSONnode, DOMDocument) {
		var elem = DOMDocument.createElement("div");
		elem.classList.add("elem-group", "elem-inspector");
		if (JSONnode.name) {
			elem.classList.add("elem-group-" + JSONnode.name);
		}
		return elem;
	}

	function createBox(JSONnode, DOMDocument) {
		var wrapperElem = DOMDocument.createElement("div");
		wrapperElem.classList.add("elem-box", "elem-inspector");
		// add margin as a padding to the wrapper element to prevent margin collapse
		addProps.addPadding(wrapperElem, JSONnode.margin);
		var elem = DOMDocument.createElement("div");
		addProps.addBackground(elem, JSONnode.background);
		addProps.addPadding(elem, JSONnode.padding);
		addProps.addBorder(elem, JSONnode.border);
		if (JSONnode.hideOnMobile && JSONnode.hideOnMobile !== "false") {
			wrapperElem.classList.add("hide-on-tablet");
		}
		if (JSONnode.hideOnDesktop && JSONnode.hideOnDesktop !== "false") {
			wrapperElem.classList.add("hide-on-desktop");
		}
		wrapperElem.appendChild(elem);
		return {
			outer: wrapperElem,
			inner: elem
		};
	}

	function createText(JSONnode, DOMDocument) {
		var textElem = subUnitFactory.createTextElem(JSONnode.text, DOMDocument);
		textElem.classList.add("elem-textbox", "elem-inspector");
		addProps.addPadding(textElem, JSONnode.spacing);
		helpers.handleOldTypography(JSONnode);
		addProps.addTypography(textElem, JSONnode.text, JSONnode.typography);
		return textElem;
	}

	function createImage(JSONnode, DOMDocument, manageBuild) {
		var outerWrapperElem = DOMDocument.createElement("div");
		outerWrapperElem.classList.add("elem-image", "elem-inspector");
		var wrapperElem = DOMDocument.createElement("div");
		outerWrapperElem.appendChild(wrapperElem);
		var imageElem = subUnitFactory.createImageElem(JSONnode.src, JSONnode.altText, JSONnode.align, DOMDocument, manageBuild);
		if (JSONnode.link !== "") {
			var linkElem = subUnitFactory.createLinkElem(JSONnode.link, DOMDocument);
			wrapperElem.appendChild(linkElem);
			linkElem.appendChild(imageElem);
		} else {
			wrapperElem.appendChild(imageElem);
		}
		addProps.addPadding(wrapperElem, JSONnode.padding);
		addProps.addPadding(outerWrapperElem, JSONnode.margin);
		addProps.addBackground(wrapperElem, JSONnode.background);

		var width = JSONnode.width - JSONnode.padding.left - JSONnode.padding.right;
		var height = JSONnode.height - JSONnode.padding.top - JSONnode.padding.bottom;
		var className = "im" + (width + "").replace(".","-") + "x" + (height + "").replace(".","-");
		if ((JSONnode.fullWidthOnMobile && JSONnode.fullWidthOnMobile !== "false") || JSONnode.width > 241) {
			className += "-full-width-on-mobile";
			styles.tablet["."+className] = {
				"width": "100%"
			};
		} else {
			styles.tablet["."+className] = {
				"max-width": width + "px",
				"width": "100%"
			};
		}
		styles.desktop["."+className] = {
			"max-width": width + "px",
			"max-height": height + "px",
			"width": "100%"
		};
		imageElem.classList.add(className, "block-image");
		return outerWrapperElem;
	}

	function createButton(JSONnode, DOMDocument) {
		var wrapperElem = DOMDocument.createElement("div");
		wrapperElem.classList.add("elem-button", "elem-inspector");
		var textElem = subUnitFactory.createTextElem(JSONnode.text, DOMDocument);
		var linkElem = subUnitFactory.createLinkElem(JSONnode.href, DOMDocument);
		wrapperElem.appendChild(linkElem);
		linkElem.appendChild(textElem);
		addProps.addPadding(textElem, JSONnode.padding);
		addProps.addPadding(wrapperElem, JSONnode.margin);
		addProps.addBorder(textElem, JSONnode.border);
		addProps.addBackground(textElem, JSONnode.background);
		helpers.handleOldTypography(JSONnode);
		addProps.addTypography(textElem, JSONnode.text, JSONnode.typography);
		wrapperElem.classList.add(JSONnode.align + "-block");
		textElem.classList.add("button-inner");
		if (JSONnode.fullWidthOnMobile && JSONnode.fullWidthOnMobile !== "false") {
			wrapperElem.classList.add("full-width-button");
			textElem.classList.add("button-inner-inherit-size-on-tablet");
		}
		if (JSONnode.sizeType === "FIXED") {
			var width = JSONnode.width;
			var className = "btn" + width;
			styles.global["."+className] = {
				"width": width + "px"
			};
			wrapperElem.classList.add(className);
			textElem.classList.add("button-inner-inherit-size");
		}
		return wrapperElem;
	}

	function createDivider(JSONnode, DOMDocument) {
		var elem = DOMDocument.createElement("div");
		elem.classList.add("elem-divider", "elem-inspector");
		addProps.addBackground(elem, JSONnode.background);
		addProps.addMargin(elem, JSONnode.spacing);
		addProps.addDividerLine(elem, JSONnode.lineStyle);
		return elem;
	}

	function createMulticolumn(JSONnode, DOMDocument) {
		var elem = DOMDocument.createElement("div");
		if (JSONnode.hideOnMobile && JSONnode.hideOnMobile !== "false") {
			elem.classList.add("hide-on-tablet");
		}
		return elem;
	}

	function createColumn(JSONnode, DOMDocument) {
		var elem = DOMDocument.createElement("div");
		elem.classList.add("column");
		elem.classList.add("elem-column", "elem-inspector");
		var columnClass = "col" + (JSONnode.width + "").replace(".", "-");
		if (JSONnode.reorderOnMobile && JSONnode.reorderOnMobile !== "false") {
			columnClass += "reorder";
			styles.desktop["." + columnClass] = {
				"width": JSONnode.width + "%"
			};
			styles.tablet["." + columnClass] = {
				"width": "100%"
			};
		} else {
			columnClass += "fixed";
			if (JSONnode.diffOnMobile) {
				columnClass += "-diff";
				styles.desktop["." + columnClass] = {
					"width": JSONnode.width + "%"
				};
				styles.tablet["." + columnClass] = {
					"width": JSONnode.mobileWidth + "%"
				};
			} else {
				styles.global["." + columnClass] = {
					"width": JSONnode.width + "%"
				};
			}
		}
		elem.classList.add(columnClass);
		if (JSONnode.hideOnMobile && JSONnode.hideOnMobile !== "false") {
			elem.classList.add("hide-on-tablet");
		}
		return elem;
	}

	return {
		createRoot: createRoot,
		createFullWidthContainer: createFullWidthContainer,
		createLeftFullWidthContainer: createLeftFullWidthContainer,
		createRightFullWidthContainer: createRightFullWidthContainer,
		createTwoCellFullWidthContainer: createTwoCellFullWidthContainer,
		createBox: createBox,
		createText: createText,
		createImage: createImage,
		createButton: createButton,
		createMulticolumn: createMulticolumn,
		createColumn: createColumn,
		createDivider: createDivider,
		createGroup: createGroup
	};
};
